//Copyright (c) 1997 The Open Group
//
//Permission to use, copy, modify, and distribute this software and its documentation for
//any purpose and without fee is hereby granted, provided that the above copyright notice
//appears in all copies and that both the copyright notice and this permission notice
//appear in supporting documentation, and that the name of ("TOG") or The Open Group not
//be used in advertising or publicity pertaining to distribution of the software without
//specific, written prior permission.
//
//THE OPEN GROUP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT
//SHALL TOG BE LIABLE FOR ANY SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT,
//NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//PERFORMANCE OF THIS SOFTWARE. 

/*
Cachable

by William E. la Forge
   http://www.osf.org/~laforge
   laforge@osf.org
*/

package ORG.opengroup.misc;

/**
 * A cachable item is retained as long as it is used occasionally.
 * @see tog.ri.moa.Cache
 */
public interface Cachable
{
/**
 * Returns time of last use in milliseconds GMT.
 * @see java.lang.System#currentTimeMillis
 */
	public long lastUsed();

/**
 * Returns the number of milliseconds the item is to be
 * retained after its last use.
 */
	public long retain();

/**
 * Release the item, it is not in active use.
 */
	public void release();

/**
 * Set the entry for the cachable, needed for remove; 
 * this field is meaningful only inside the Cache object.
 */
	public void setEntry(int entry);

/**
 * Get the entry for the cachable, needed for remove;
 * this field is meaningful only inside the Cache object.
 */
	public int getEntry();
}
