//Copyright (c) 1997 The Open Group
//
//Permission to use, copy, modify, and distribute this software and its documentation for
//any purpose and without fee is hereby granted, provided that the above copyright notice
//appears in all copies and that both the copyright notice and this permission notice
//appear in supporting documentation, and that the name of ("TOG") or The Open Group not
//be used in advertising or publicity pertaining to distribution of the software without
//specific, written prior permission.
//
//THE OPEN GROUP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT
//SHALL TOG BE LIABLE FOR ANY SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT,
//NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//PERFORMANCE OF THIS SOFTWARE. 

/*
Cache

by William E. la Forge
   http://www.osf.org/~laforge
   laforge@osf.org
*/

package ORG.opengroup.misc;

import java.util.*;

/**
* This object is used for timing purposes. It keeps a list of
* the objects to be times
*/
public class Cache extends Thread
{
	private Vector items[];
	private long gran;
	private int ext;
	private int next;
	private long tm;
	private int entries;

    /**
     * @param granularity length of sleep of Cache.
     * @param extend size of Cache array.
     */
	public Cache(long granularity, int extent)
	{
		setDaemon(true);
		gran=granularity;
		ext=extent;
		next=0;
		tm=System.currentTimeMillis();
		items=new Vector[extent];
		int i;
		for (i=0;i<ext;++i)
			items[i]=new Vector();
	}

    /**
    * Returns the current time
    */
	public long getTime()
	{
		return tm;
	}

    /**
    * Performs timing operations
    */
	public void run()
	{
		while (true)
		{
			try
			{
				sleep(gran);
			} catch (InterruptedException ie)
			{
			}
			Vector v=null;
			int s=0;
			synchronized(this)
			{
				tm=System.currentTimeMillis();
				v=items[next];
				s=v.size();
				if (s>0)
					items[next]=new Vector();
				next=(next+1)%ext;
			}
			int i;
			for (i=0;i<s;++i)
			{
				Cachable c=(Cachable)v.elementAt(i);
				synchronized(c)
				{
					long e=c.lastUsed()+c.retain();
					if (e<tm)
					{
						entries--;
						c.release();
					}
					else
						add(c);
				}
			}
		}
	}

    /**
    * Adds the object which needs a timer to the Cache
    * @param c the object to be timed
    */
	public synchronized void add(Cachable c)
	{
		long e=c.lastUsed()+c.retain();
		if(e<tm)
			e=tm;
		int i=(int)((e-tm)/gran);
		if (i>(ext-1))
			i=ext-1;
		i=(i+next)%ext;
		Vector v=items[i];
		v.addElement(c);
		c.setEntry(i); // added setting of the entry field in cachable
		entries++;
	}

    /**
    * Reschedules an object which needs a timer
    * @param c the object to be timed
    */
	public synchronized void reschedule(Cachable c)
	{
		if (items[c.getEntry()].removeElement(c))
			add(c);
	}

    /**
    * Removes an object which needs a timer
    * @param c the Cachable object to be removed
    */
	public void remove (Cachable c) {

		Vector v=null;
		int s=0;
		synchronized(this)
		{
			tm=System.currentTimeMillis();
			v=items[c.getEntry()];
			s=v.size();
			if (s>0)
				items[c.getEntry()]=new Vector();
			// if s=0 the cache entry might have expired
		}
		int i;
		for (i=0;i<s;++i)
		{
			Cachable cx=(Cachable)v.elementAt(i);
			synchronized(cx)
			{
				if (cx==c) {
					cx.release();
					entries--;
				}
				else
					add(cx);
			}
		}
	}

	/**
	 * Return the current number of entries of the cache. Even though
	 * the entries is maintained in the Cache object, it is up to the
	 * object that adds/removes entries to act upon this information.
	 */
	public int getEntries() {
		return entries;
	}
}
