//Copyright (c) 1997 The Open Group
//
//Permission to use, copy, modify, and distribute this software and its documentation for
//any purpose and without fee is hereby granted, provided that the above copyright notice
//appears in all copies and that both the copyright notice and this permission notice
//appear in supporting documentation, and that the name of ("TOG") or The Open Group not
//be used in advertising or publicity pertaining to distribution of the software without
//specific, written prior permission.
//
//THE OPEN GROUP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT
//SHALL TOG BE LIABLE FOR ANY SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT,
//NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//PERFORMANCE OF THIS SOFTWARE. 

/* 
 * ORG.opengroup.moajar.MoaJar version 0.1
 * by Bill la Forge
 * http://www.opengroup.org/~laforge
 */

package ORG.opengroup.moajar;
import ORG.opengroup.awt.*;
import ORG.opengroup.jar.*;
import ORG.opengroup.misc.*;
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.beans.*;

public class MoaJar extends Applet
{
	private static Frame f;
	private static String lck=new String();
	private static String flck=new String();

	static class TextFieldDialog extends MsgFieldDialog
	{
		public TextFieldDialog(
			Frame parent, 
			String title, 
			String message,
			String txt,
			int col,
			String one)
		{
			super(parent,title,200,200,message,txt,col,0,one+"\nCancel");
		}

		public TextFieldDialog(
			Frame parent, 
			String title, 
			String message,
			String txt,
			int col,
			String one,
			String two)
		{
			super(parent,title,200,200,message,txt,col,0,one+"\n"+two);
		}
	}

	static class MsgDialog extends TextAreaDialog
	{
		public MsgDialog(Frame parent, String title, String message)
		{
			super(parent,title,200,200,message,"Continue");
		}

		public MsgDialog(Frame parent, String title, String message, String buttons)
		{
			super(parent,title,200,200,message,buttons);
		}
	}

	static class JarFrame extends Frame
	{
		MenuBar menuBar;
		Menu jarMenu, fileMenu, helpMenu;
		MenuItem jarNewItem, jarOpenItem, jarSaveItem, jarSaveAsItem, jarExitItem;
		MenuItem fileIncludeItem, fileDeleteItem, fileExtractItem, fileRenameItem,
			fileCopyItem, fileInstantiateItem, fileEditItem;
		MenuItem helpTopicsItem, helpAboutItem;

		String fileName=new String("");
		String directoryName=null;
		String fileDirectoryName=null;
		JarFile jarFile=new JarFile();
		boolean modified=false;
		List fileList;
		TextArea propertyList;
		String propertyText="";
		Manifest manifest;
		ManifestSection manifestSection;
		String selectedFile;
		
		class Unimplemented implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				updateProperties();
				synchronized(lck)
				{
					Dialog d=new MoaJar.MsgDialog(MoaJar.JarFrame.this,
						"Unimplemented",
						"This selection has not yet been implemented.");
					d.show();
					return;
				}
			}
		}
		
		class FileEdit implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				updateProperties();
				String nsf;
				synchronized(flck)
				{
					nsf=selectedFile;
				}
				synchronized(lck)
				{
					if (nsf==null||nsf.startsWith("META-INF/"))
					{
						Dialog d=new MoaJar.MsgDialog(
							MoaJar.JarFrame.this,
							"Unselected",
							"No file has been selected.");
						d.show();
						return;
					}
					if (!nsf.endsWith(".ser"))
					{
						Dialog d=new MoaJar.MsgDialog(MoaJar.JarFrame.this,
							"Serialized Object Required",
							"The file must end with \".ser\".");
						d.show();
						return;
					}
					nsf=nsf.substring(0,nsf.length()-4);
					modified=true;
					try
					{
						Object obj=jarFile.getObject(nsf);
						PropertySheetDialog d=new PropertySheetDialog(
							MoaJar.JarFrame.this,
							nsf,
							200,200,
							"Continue",
							obj);
						d.show();
						jarFile.putObject(nsf,obj);
					}catch (Exception ex)
					{
						Dialog dx=new MoaJar.MsgDialog(MoaJar.JarFrame.this,
							"Error",
							ex.toString());
						dx.show();
						nsf=null;
					}
				}
			}
		}
		
		class FileInstantiate implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				updateProperties();
				String nsf;
				String name;
				synchronized(flck)
				{
					nsf=selectedFile;
				}
				synchronized(lck)
				{
					if (nsf==null)
						nsf="";
					else if (nsf.endsWith(".class"))
						nsf=nsf.substring(0,nsf.length()-6);
					else if (nsf.endsWith(".ser"))
						nsf=nsf.substring(0,nsf.length()-4);
					else
						nsf="";
					nsf=nsf.replace('/','.');
					TextFieldDialog d=new MoaJar.TextFieldDialog(
						MoaJar.JarFrame.this,
						"Instantiate",
						"Enter the class name:",
						nsf,
						60,
						"OK");
					d.show();
					if (d.getSelection()!=0)
						return;
					name=d.getText();
					if (name.length()==0)
					{
						return;
					}
					nsf=name.replace('.','/')+".ser";
					if (jarFile.get(nsf)!=null)
					{
						MoaJar.MsgDialog td=new MoaJar.MsgDialog(
							MoaJar.JarFrame.this,
							"Warning",
							"File "+nsf+" exists. Should it be overwritten?",
							"Overwrite\nDon't");
						td.show();
						if(td.getSelection()!=0)
							return;
						jarFile.remove(nsf);
					}
					modified=true;
					try
					{
						Object obj=jarFile.getObject(name);
						jarFile.putObject(name,obj);
					}catch (Exception ex)
					{
						Dialog dx=new MoaJar.MsgDialog(MoaJar.JarFrame.this,
							"Error",
							ex.toString());
						dx.show();
						nsf=null;
					}
				}
				synchronized(flck)
				{
					selectedFile=nsf;
				}
				rebuild();
			}
		}
		
		class FileCopy implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				updateProperties();
				String nsf;
				String name;
				synchronized(flck)
				{
					nsf=selectedFile;
				}
				synchronized(lck)
				{
					if (nsf==null||nsf.startsWith("META-INF/"))
					{
						Dialog d=new MoaJar.MsgDialog(MoaJar.JarFrame.this,
							"Unselected",
							"No file has been selected.");
						d.show();
						return;
					}
					TextFieldDialog d=new MoaJar.TextFieldDialog(
						MoaJar.JarFrame.this,
						"Rename",
						"Enter the new name:",
						nsf,
						60,
						"OK");
					d.show();
					if (d.getSelection()!=0)
						return;
					name=d.getText();
					if (nsf.equals(name))
						return;
					if (name.length()==0 || name.startsWith("META-INF/"))
					{
						Dialog de=new MoaJar.MsgDialog(MoaJar.JarFrame.this,
							"Error",
							"Not a valid name.");
						de.show();
						return;
					}
					if (jarFile.get(name)!=null)
					{
						MoaJar.MsgDialog td=new MoaJar.MsgDialog(
							MoaJar.JarFrame.this,
							"Warning",
							"File "+name+" exists. Should it be overwritten?",
							"Overwrite\nDon't");
						td.show();
						if(td.getSelection()!=0)
							return;
					}
					modified=true;
					byte data[]=jarFile.get(nsf);
					jarFile.put(name,data);
				}
				synchronized(flck)
				{
					selectedFile=name;
				}
				rebuild();
			}
		}
		
		class FileRename implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				updateProperties();
				String nsf;
				String name;
				synchronized(flck)
				{
					nsf=selectedFile;
				}
				synchronized(lck)
				{
					if (nsf==null||nsf.startsWith("META-INF/"))
					{
						Dialog d=new MoaJar.MsgDialog(MoaJar.JarFrame.this,
							"Unselected",
							"No file has been selected.");
						d.show();
						return;
					}
					TextFieldDialog d=new MoaJar.TextFieldDialog(
						MoaJar.JarFrame.this,
						"Rename",
						"Enter the new name:",
						nsf,
						60,
						"OK");
					d.show();
					if (d.getSelection()!=0)
						return;
					name=d.getText();
					if (nsf.equals(name))
						return;
					if (name.length()==0 || name.startsWith("META-INF/"))
					{
						Dialog de=new MoaJar.MsgDialog(MoaJar.JarFrame.this,
							"Error",
							"Not a valid name.");
						de.show();
						return;
					}
					if (jarFile.get(name)!=null)
					{
						MoaJar.MsgDialog td=new MoaJar.MsgDialog(
							MoaJar.JarFrame.this,
							"Warning",
							"File "+name+" exists. Should it be overwritten?",
							"Overwrite\nDon't");
						td.show();
						if(td.getSelection()!=0)
							return;
						jarFile.remove(name);
					}
					modified=true;
					manifestSection=manifest.force(nsf);
					manifestSection.add("Name",name);
					byte data[]=jarFile.get(nsf);
					jarFile.remove(nsf);
					jarFile.put(name,data);
				}
				synchronized(flck)
				{
					selectedFile=name;
				}
				rebuild();
			}
		}
		
		class Topics implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				updateProperties();
				synchronized(lck)
				{
					String text;
					Dialog d=new TopicsDialog(
						MoaJar.JarFrame.this,200,0,this);
					d.show();
					return;
				}
			}
		}
		
		class About implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				updateProperties();
				synchronized(lck)
				{
					String text;
					try
					{
						text=new String(Reads.read(this.getClass().getResourceAsStream("notice.txt")));
					} catch (Exception eeee)
					{
						text="Unable to read notice.txt";
					}
					Dialog d=new MoaJar.MsgDialog(
						MoaJar.JarFrame.this,
						"About MoaJar",text);
					d.show();
					return;
				}
			}
		}
		
		class FileExtract implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				updateProperties();
				String nsf;
				synchronized(flck)
				{
					nsf=selectedFile;
				}
				synchronized(lck)
				{
					if (nsf==null)
					{
						Dialog d=new MoaJar.MsgDialog(MoaJar.JarFrame.this,
							"Unselected",
							"No file has been selected.");
						d.show();
						return;
					}
					FileDialog fd=new FileDialog(MoaJar.JarFrame.this,
						"Extract File", FileDialog.SAVE);
					String fnam=nsf.replace('/','.');
					fd.setFile(fnam);
					if (fileDirectoryName==null)
						fileDirectoryName=directoryName;
					if (fileDirectoryName!=null)
						fd.setDirectory(fileDirectoryName);
					fd.show();
					if (fd.getFile()!=null)
					{
						try
						{
							byte data[]=jarFile.get(nsf);
							FileOutputStream fos=new FileOutputStream
								(fd.getDirectory()+fd.getFile());
							fos.write(data);
							fos.close();
						} catch (IOException ex)
						{
							Dialog d=new MoaJar.MsgDialog(
								MoaJar.JarFrame.this,
								"Error",
								"Unable to save "+
								fd.getDirectory()+fd.getFile());
							d.show();
							return;
						}
					}
				}
			}
		}
		
		class FileDelete implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				updateProperties();
				String nsf;
				synchronized(flck)
				{
					nsf=selectedFile;
				}
				synchronized(lck)
				{
					if (nsf==null)
					{
						Dialog d=new MoaJar.MsgDialog(MoaJar.JarFrame.this,
							"Unselected",
							"No file has been selected.");
						d.show();
						return;
					}
					jarFile.remove(nsf);
					modified=true;
				}
				synchronized(flck)
				{
					selectedFile=null;
				}
				rebuild();
			}
		}

		class FileInclude implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				updateProperties();
				String selFile;
				synchronized(lck)
				{
					FileDialog fd=new FileDialog(MoaJar.JarFrame.this,
						"Include File", FileDialog.LOAD);
					if (fileDirectoryName==null)
						fileDirectoryName=directoryName;
					if (fileDirectoryName!=null)
						fd.setDirectory(fileDirectoryName);
					fd.show();
					selFile=fd.getFile();
					if (selFile==null)
						return;
					try
					{
						if (jarFile.get(selFile)!=null)
						{
							MoaJar.MsgDialog td=new MoaJar.MsgDialog(
								MoaJar.JarFrame.this,
								"Warning",
								"File exists. Should it be overwritten?",
								"Overwrite\nDon't");
							td.show();
							if(td.getSelection()!=0)
							{
								return;
							}
							jarFile.remove(selFile);
						}
						jarFile.putFile(fd.getDirectory()+selFile, selFile);
					}catch (Exception ex)
					{
						Dialog d=new MoaJar.MsgDialog(MoaJar.JarFrame.this,
							"Error",
							"Unable to open "+selFile);
						d.show();
						return;
					}
					fileDirectoryName=fd.getDirectory();
					modified=true;
				}
				synchronized(flck)
				{
					selectedFile=selFile;
				}
				rebuild();
			}
		}

		class FileIL implements ItemListener
		{
			public void itemStateChanged(ItemEvent ie)
			{
				String nsf=fileList.getSelectedItem();
				synchronized(flck)
				{
					if (nsf==null && selectedFile==null)
					{
						return;
					}
					if (nsf!=null && nsf.equals(selectedFile))
					{
						return;
					}
				}
				updateProperties();
				synchronized(lck)
				{
					propertyText="";
					manifest=jarFile.getManifest();
					manifestSection=null;
					if (manifest==null)
						nsf=null;
					else
					{
						if (nsf!=null && !nsf.equals(""))
						{
							manifestSection=manifest.get(nsf);
							if (manifestSection!=null)
							{
								String pa[]=manifestSection.get();
								int len=pa.length;
								int i;
								for (i=0;i<len;++i)
									try
									{
										String lead=pa[i].substring(0,5);
										if (!lead.equalsIgnoreCase("Name:"))
											propertyText+=pa[i]+"\n";
									} catch (StringIndexOutOfBoundsException e)
									{
										propertyText+=pa[i]+"\n";
									}
							}
						}
					}
					propertyList.replaceRange(propertyText,0,1000);
				}
				synchronized(flck)
				{
					selectedFile=nsf;
				}
				rebuild();
			}
		}
		
		class JarSaveAs implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				updateProperties();
				synchronized(lck)
				{
					try
					{
						if (saveJarAs())
							modified=false;
					} catch (Exception ex)
					{
						Dialog d=new MoaJar.MsgDialog(
							MoaJar.JarFrame.this,
							"Error",
							"Unable to save "+
							directoryName+fileName);
						d.show();
					}
				}
			}
		}
		
		class JarSave implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				updateProperties();
				synchronized(lck)
				{
					try
					{
						if (saveJar())
							modified=false;
					}catch (Exception ex)
					{
						Dialog d=new MoaJar.MsgDialog(MoaJar.JarFrame.this,
							"Error",
							"Unable to save "+
							directoryName+fileName);
						d.show();
					}
				}
			}
		}
		
		class JarNew implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				if (!saveJarMaybe())
					return;
				updateProperties();
				synchronized(lck)
				{
					fileName=new String("");
					jarFile=new JarFile();
					manifestSection=null;
					modified=false;
					setTitle("MoaJar - ");
				}
				synchronized(flck)
				{
					selectedFile=null;
				}
				rebuild();
			}
		}
		
		class JarOpen implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				if (!saveJarMaybe())
					return;
				synchronized(lck)
				{
					FileDialog fd=new FileDialog(MoaJar.JarFrame.this,
						"Open Jar File", FileDialog.LOAD);
					if (!fileName.equals(""))
						fd.setFile(fileName);
					if (directoryName!=null)
						fd.setDirectory(directoryName);
					fd.show();
					if (fd.getFile()==null)
						return;
					try
					{
						jarFile=new JarFile(
							fd.getDirectory()+fd.getFile());
					}catch (Exception ex)
					{
						Dialog d=new MoaJar.MsgDialog(MoaJar.JarFrame.this,
							"Error",
							"Unable to open "+
							fd.getDirectory()+fd.getFile());
						d.show();
						return;
					}
					manifest=jarFile.getManifest();
					manifestSection=null;
					fileName=fd.getFile();
					directoryName=fd.getDirectory();
					modified=false;
					setTitle("MoaJar - "+
						directoryName+
						fileName);
				}
				synchronized(flck)
				{
					selectedFile=null;
				}
				rebuild();
			}
		}
		
		class JarExit implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				zap();
			}
		}
	
		class Zap extends WindowAdapter
		{
			public void windowClosing(WindowEvent e)
			{
				zap();
			}
		}
	
		public JarFrame(String title)
		{
			super(title);
			addWindowListener(new Zap());
			setLocation(100,100);
			setSize(600,800);
			menuBar=new MenuBar();
			setMenuBar(menuBar);
	
			jarMenu=new Menu("Jar");
			jarNewItem=new MenuItem("New");
			jarNewItem.addActionListener(new JarNew());
			jarMenu.add(jarNewItem);
			jarOpenItem=new MenuItem("Open");
			jarOpenItem.addActionListener(new JarOpen());
			jarMenu.add(jarOpenItem);
			jarSaveItem=new MenuItem("Save");
			jarSaveItem.addActionListener(new JarSave());
			jarMenu.add(jarSaveItem);
			jarSaveAsItem=new MenuItem("Save As");
			jarSaveAsItem.addActionListener(new JarSaveAs());
			jarMenu.add(jarSaveAsItem);
			jarExitItem=new MenuItem("Exit");
			jarExitItem.addActionListener(new JarExit());
			jarMenu.add(jarExitItem);
			menuBar.add(jarMenu);
	
			fileMenu=new Menu("File");
			fileIncludeItem=new MenuItem("Include");
			fileIncludeItem.addActionListener(new FileInclude());
			fileMenu.add(fileIncludeItem);
			fileDeleteItem=new MenuItem("Delete");
			fileDeleteItem.addActionListener(new FileDelete());
			fileMenu.add(fileDeleteItem);
			fileExtractItem=new MenuItem("Extract");
			fileExtractItem.addActionListener(new FileExtract());
			fileMenu.add(fileExtractItem);
			fileRenameItem=new MenuItem("Rename");
			fileRenameItem.addActionListener(new FileRename());
			fileMenu.add(fileRenameItem);
			fileCopyItem=new MenuItem("Copy");
			fileCopyItem.addActionListener(new FileCopy());
			fileMenu.add(fileCopyItem);
			fileInstantiateItem=new MenuItem("Instantiate");
			fileInstantiateItem.addActionListener(new FileInstantiate());
			fileMenu.add(fileInstantiateItem);
			fileEditItem=new MenuItem("Edit");
			fileEditItem.addActionListener(new FileEdit());
			fileMenu.add(fileEditItem);
			menuBar.add(fileMenu);
	
			helpMenu=new Menu("Help");
			helpTopicsItem=new MenuItem("Topics");
			helpTopicsItem.addActionListener(new Topics());
			helpMenu.add(helpTopicsItem);
			helpAboutItem=new MenuItem("About");
			helpAboutItem.addActionListener(new About());
			helpMenu.add(helpAboutItem);
			menuBar.add(helpMenu);
			menuBar.setHelpMenu(helpMenu);

			setLayout(new BorderLayout(5,5));
			Panel lp=new Panel();
			lp.setLayout(new BorderLayout(0,0));
			add("North",lp);
			Label ll=new Label(
"Files in the Jar                                                               ");
			lp.add("North",ll);
			fileList=new List(15);
			fileList.addItemListener(new FileIL());
			lp.add("South",fileList);
			Panel rp=new Panel();
			rp.setLayout(new BorderLayout(0,0));
			add("South",rp);
			Label rl=new Label(
"File Attributes                                                                ");
			rp.add("North",rl);
			propertyList=new TextArea("",10,72);
			rp.add("South",propertyList);
			pack();
		}

		public void rebuild()
		{
			String nsf;
			synchronized(flck)
			{
				nsf=selectedFile;
			}
			synchronized(lck)
			{
				boolean oldVisible=fileList.isVisible();
				fileList.setVisible(false);
				fileList.removeAll();
				Enumeration en=jarFile.resources();
				int ndx=0;
				int selndx=-1;
				while(en.hasMoreElements())
				{
					String nxtfle=(String)en.nextElement();
					fileList.add(nxtfle);
					if (nsf!=null && 
							nsf.equals(nxtfle))
						selndx=ndx;
					++ndx;
				}
				propertyText="";
				if (selndx!=-1)
				{
					fileList.select(selndx);
					manifest=jarFile.getManifest();
					manifestSection=manifest.get(nsf);
					if (manifestSection!=null)
					{
						String pa[]=manifestSection.get();
						int len=pa.length;
						int i;
						for (i=0;i<len;++i)
						{
							try
							{
								String lead=pa[i].substring(0,5);
								if (!lead.equalsIgnoreCase("Name:"))
									propertyText+=pa[i]+"\n";
							} catch (StringIndexOutOfBoundsException e)
							{
								propertyText+=pa[i]+"\n";
							}
						}
					}
				}
				propertyList.replaceRange(propertyText,0,1000);
				invalidate();
				fileList.setVisible(oldVisible);
			}
		}

		public void updateProperties()
		{
			String nsf;
			synchronized(flck)
			{
				nsf=selectedFile;
			}
			synchronized(lck)
			{
				String t=propertyList.getText();
				if (t.equals(propertyText))
					return;
				if (nsf==null)
					return;
				if (nsf.startsWith("META-INF/"))
					return;
				modified=true;
				Vector s=new Vector();
				s.addElement("Name: "+nsf);
				int l=t.length();
				int f=0;
				int n=0;
				while(f<l)
				{
					n=t.indexOf('\n',f);
					if (n==-1)
						n=l;
					String x=t.substring(f,n);
					try
					{
						String lead=x.substring(0,5);
						if (!lead.equalsIgnoreCase("Name:"))
							s.addElement(x);
					} catch (StringIndexOutOfBoundsException e)
					{
						s.addElement(x);
					}
					f=n+1;
				}
				int sz=s.size();
				String ln[]=new String[sz];
				s.copyInto(ln);
				manifest=jarFile.getManifest();
				manifest.add(new ManifestSection(ln));
			}
		}

		public boolean saveJar()
			throws Exception
		{
			synchronized(lck)
			{
				if (fileName.equals(""))
					return saveJarAs();
				jarFile.save();
				return true;
			}
		}

		public boolean saveJarAs()
			throws Exception
		{
			synchronized(lck)
			{
				FileDialog fd=new FileDialog(MoaJar.JarFrame.this,
					"Save Jar File", FileDialog.SAVE);
				if (!fileName.equals(""))
					fd.setFile(fileName);
				if (directoryName!=null)
					fd.setDirectory(directoryName);
				fd.show();
				if (fd.getFile()==null)
					return false;
				fileName=fd.getFile();
				directoryName=fd.getDirectory();
				setTitle("MoaJar - "+
					directoryName+
					fileName);
				modified=true;
				jarFile.saveAs(directoryName+fileName);
				modified=false;
				return true;
			}
		}

		public boolean saveJarMaybe()
		{
			updateProperties();
			synchronized(lck)
			{
				if (modified)
				{
					MoaJar.MsgDialog td=new MoaJar.MsgDialog(this,"Save Jar File?",
						"Should the Jar File be saved?","Save\nDon't");
					td.show();
					if(td.getSelection()==0)
					{
						try
						{
							if (!saveJar())
								throw new Exception();
						}catch (Exception ex)
						{
							MoaJar.MsgDialog d=new MoaJar.MsgDialog(this,
								"Error",
								"Unable to save "+
								directoryName+fileName,"Continue\nCancel");
							d.show();
							return d.getSelection()==0;
						}
					}
				}
				return true;
			}
		}
	
		public void zap()
		{
			if (!saveJarMaybe())
				return;
			//synchronized(lck)
			{
				System.exit(0);
			}
		}
	}

	public void start()
	{
		f=new JarFrame("MoaJar - ");
		f.show();
	}

	public static void main(String argv[])
	{
		Beans.setDesignTime(true);
		f=new JarFrame("MoaJar - ");
		f.show();
	}
}
